import subprocess
import tempfile
import os
import sys

class Mountpoint(tempfile.TemporaryDirectory):
    pass


class PartitionMounter:
    def __init__(self, img_file, partition_number):
        self.img_file = img_file
        self.partition_number = partition_number
        self.mountpoint = None
        self.loop_device = None

    def setup_loop_device(self):
        self.loop_device = subprocess.check_output(['losetup', '--show', '-fP', self.img_file], text=True).strip()
        print(f"Loop device created: {self.loop_device}")

    def mount_partition(self):
        if self.loop_device is None:
            self.setup_loop_device()

        # Use partprobe to make the kernel aware of the partition
        subprocess.run(['partprobe', self.loop_device], check=True)

        # Calculate partition path
        partition_path = f"{self.loop_device}p{self.partition_number}"

        # Create a temporary directory to mount the partition
        self.mountpoint = Mountpoint()

        # Mount the partition to the temporary directory
        subprocess.run(['mount', partition_path, self.mountpoint.name], check=True)

    def cleanup(self):
        if self.mountpoint:
            subprocess.run(['umount', self.mountpoint.name], check=True)
            self.mountpoint.cleanup()

        if self.loop_device:
            subprocess.run(['losetup', '-d', self.loop_device], check=True)

    def __enter__(self):
        self.mount_partition()
        return self.mountpoint.name

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cleanup()
