from pathlib import Path
from typing import Tuple

import os
import shutil
import subprocess

import utils.fdisk

from .mount import PartitionMounter


class VMEditor:
    def __init__(self, source: os.PathLike, destination: os.PathLike):
        self.source_path = Path(source)
        self.destination_path = Path(destination)
        self.system_partition: Tuple[str, int] = None
        self.mount: PartitionMounter = None

    def create_copy(self):
        shutil.copytree(self.source_path, self.destination_path)

    def update_filenames(self):
        for file in self.destination_path.iterdir():
            if self.source_path.name in file.name:
                file.rename(
                    self.destination_path
                    / file.name.replace(
                        self.source_path.name, self.destination_path.name
                    )
                )

    def update_file_contents(self):
        vmxpath = self.destination_path / f"{self.destination_path.name}.vmx"
        content = ""

        with (vmxpath).open("r") as vmx_in:
            content = vmx_in.read()

        content = content.replace(self.source_path.name, self.destination_path.name)

        with vmxpath.open("w") as vmx_out:
            vmx_out.write(content)

        for vmdk in self.destination_path.glob("*.vmdk"):
            if vmdk.stat().st_size < 1e5:
                with vmdk.open("r") as vmdk_in:
                    content = vmdk_in.read()

                content = content.replace(
                    self.source_path.name, self.destination_path.name
                )

                with vmdk.open("w") as vmdk_out:
                    content = vmdk_out.write(content)

    def vmdk_to_img(self):
        for vmdk in self.destination_path.glob("*.vmdk"):
            if vmdk.stem.endswith("-flat"):
                continue

            img = self.destination_path / f"{vmdk.stem}.img"

            subprocess.run(
                [
                    "qemu-img",
                    "convert",
                    "-O",
                    "raw",
                    str(vmdk),
                    str(img),
                ],
                check=True,
            )

            vmdk.unlink()

            if (flat := self.destination_path / f"{vmdk.stem}-flat.vmdk").exists():
                flat.unlink()

    def get_partitions(self):
        files = []

        for img in self.destination_path.glob("*.img"):
            files += utils.fdisk.get_partitions(img)

        return files

    def mount_system_partition(self):
        self.mount = PartitionMounter(*self.system_partition)
        self.mount.mount_partition()
