import npyscreen

import time
import sys

from .qemu import VMEditor


class VMApp(npyscreen.NPSAppManaged):
    editor: VMEditor = None

    def onStart(self):
        self.addForm("MAIN", VMConfiguratorForm, name="VM Configurator")
        self.addForm("PARTITION", PartitionSelectorForm, name="Partition Selector")


class VMConfiguratorForm(npyscreen.FormBaseNew):
    def create(self):
        self.sourceDir = self.add(npyscreen.TitleFilename, name="Source Directory:")
        self.newName = self.add(npyscreen.TitleText, name="New Name:")
        self.button = self.add(
            npyscreen.ButtonPress, name="Start", when_pressed_function=self.on_start
        )

    def on_start(self):
        source_dir = self.sourceDir.value
        new_name = self.newName.value

        if npyscreen.notify_ok_cancel(
            f"Directory: {source_dir}\nNew Name: {new_name}",
            title="Info",
            form_color="CONTROL",
        ):
            self.process()

    def process(self):
        self.parentApp.editor = VMEditor(self.sourceDir.value, self.newName.value)

        message = "Processing:\n\n"

        npyscreen.notify_wait(
            message := message + "- Copying files\n", title="Please wait..."
        )
        # self.parentApp.editor.create_copy()

        npyscreen.notify_wait(
            message := message + "- Updating file names\n", title="Please wait..."
        )
        # self.parentApp.editor.update_filenames()

        npyscreen.notify_wait(
            message := message + "- Updating configuration file\n",
            title="Please wait...",
        )
        # self.parentApp.editor.update_file_contents()

        npyscreen.notify_wait(
            message := message + "- Converting disk to .img\n", title="Please wait..."
        )
        # self.parentApp.editor.vmdk_to_img()

        npyscreen.notify_wait(
            message + "- Preparing partition selector", title="Please wait..."
        )

        self.parentApp.switchForm("PARTITION")


class PartitionSelectorForm(npyscreen.FormBaseNew):
    def create(self):
        self.partition = self.add(npyscreen.SelectOne, values=[], max_height=10)
        self.button = self.add(
            npyscreen.ButtonPress, name="Select", when_pressed_function=self.on_select
        )

    def pre_edit_loop(self):
        self.partition.values = [
            f'{partition["device"]} – {partition["size"]} ({partition["type"]}{partition["boot"]})'
            for partition in self.parentApp.editor.get_partitions()
        ]

    def on_select(self):
        partition = self.partition.values[self.partition.value[0]]

        path = partition.split(" – ")[0]

        file = ".img".join(path.split(".img")[:-1]) + ".img"
        number = path.split(".img")[-1]

        message = "Processing:\n\n"

        npyscreen.notify_wait(
            message := message + "- Mounting system partition\n", title="Please wait..."
        )

        self.parentApp.editor.system_partition = (file, number)
        self.parentApp.editor.mount_system_partition()

        if npyscreen.notify_ok_cancel(
            f"Mountpoint: {self.parentApp.editor.mount.mountpoint.name}",
            title="Info",
            form_color="CONTROL",
        ):
            sys.exit()
